@@ -33,11 +33,13 @@ when called from application.py instead of bin/pitivi, if it has an impact.
 Package maintainers should look at the bottom section of this file.
 """
 
+import os
 import sys
 
 from gettext import gettext as _
 
 missing_soft_deps = {}
+videosink_factory = None
 
 
 def _version_to_string(version):
@@ -49,6 +51,7 @@ def _string_to_list(version):
 
 
 class Dependency(object):
+
     """
     This abstract class represents a module or component requirement.
     @param modulename: The string allowing for import or lookup of the component.
@@ -57,7 +60,8 @@ class Dependency(object):
     @param additional_message: A string that will be displayed to the user to further
       explain the purpose of the missing component.
     """
-    def __init__(self, modulename, version_required_string, additional_message=None):
+
+    def __init__(self, modulename, version_required_string=None, additional_message=None):
         self.version_required_string = version_required_string
         self.modulename = modulename
         self.satisfied = False
@@ -120,19 +124,27 @@ class Dependency(object):
 
 
 class GIDependency(Dependency):
+    def __init__(self, modulename, apiversion, version_required_string=None, additional_message=None):
+        self.__api_version = apiversion
+        super(GIDependency, self).__init__(modulename, version_required_string, additional_message)
+
     def _try_importing_component(self):
         try:
+            import gi
+            try:
+                gi.require_version(self.modulename, self.__api_version)
+            except ValueError:
+                return None
+
             __import__("gi.repository." + self.modulename)
             module = sys.modules["gi.repository." + self.modulename]
         except ImportError:
             module = None
         return module
 
-    def _format_version(self, module):
-        pass
-
 
 class ClassicDependency(Dependency):
+
     def _try_importing_component(self):
         try:
             __import__(self.modulename)
@@ -141,15 +153,14 @@ class ClassicDependency(Dependency):
             module = None
         return module
 
-    def _format_version(self, module):
-        pass
-
 
 class GstPluginDependency(Dependency):
+
     """
     Don't call check on its instances before actually checking
     Gst is importable.
     """
+
     def _try_importing_component(self):
         try:
             from gi.repository import Gst
@@ -166,16 +177,19 @@ class GstPluginDependency(Dependency):
 
 
 class GstDependency(GIDependency):
+
     def _format_version(self, module):
         return list(module.version())
 
 
-class GtkOrClutterDependency(GIDependency):
+class GtkDependency(GIDependency):
+
     def _format_version(self, module):
         return [module.MAJOR_VERSION, module.MINOR_VERSION, module.MICRO_VERSION]
 
 
 class CairoDependency(ClassicDependency):
+
     def __init__(self, version_required_string):
         ClassicDependency.__init__(self, "cairo", version_required_string)
 
@@ -192,6 +206,40 @@ def _check_audiosinks():
     return True
 
 
+def _check_videosink():
+    from gi.repository import Gst
+    from gi.repository import Gdk
+    from gi.repository import GObject
+    global videosink_factory
+
+    try:
+        # If using GdkBroadwayDisplay make sure not to try to use gtkglsink
+        # as it would segfault right away.
+        if GObject.type_is_a(Gdk.Display.get_default().__gtype__,
+                             GObject.type_from_name("GdkBroadwayDisplay")):
+            videosink_factory = Gst.ElementFactory.find("gtksink")
+            return True
+    except RuntimeError:
+        pass
+
+    if "gtkglsink" in os.environ.get("PITIVI_UNSTABLE_FEATURES", ''):
+        sink = Gst.ElementFactory.make("gtkglsink", None)
+        if not sink:
+            videosink_factory = sink.get_factory()
+        elif sink.set_state(Gst.State.READY) == Gst.StateChangeReturn.SUCCESS:
+            videosink_factory = sink.get_factory()
+            sink.set_state(Gst.State.NULL)
+        else:
+            videosink_factory = Gst.ElementFactory.find("gtksink")
+    else:
+        videosink_factory = Gst.ElementFactory.find("gtksink")
+
+    if videosink_factory:
+        return True
+
+    return False
+
+
 def _check_gst_python():
     from gi.repository import Gst
     try:
@@ -201,16 +249,27 @@ def _check_gst_python():
     return True
 
 
+class GICheck(ClassicDependency):
+    def __init__(self, version_required_string):
+        ClassicDependency.__init__(self, "gi", version_required_string)
+
+    def _format_version(self, module):
+        return list(module.version_info)
+
+
 def check_requirements():
     hard_dependencies_satisfied = True
+
     for dependency in HARD_DEPENDENCIES:
         dependency.check()
-        if not dependency.satisfied:
-            if hard_dependencies_satisfied:
-                print((_("ERROR - The following hard dependencies are unmet:")))
-                print("==================================================")
-            print(dependency)
+        if dependency.satisfied:
+            continue
+        if hard_dependencies_satisfied:
             hard_dependencies_satisfied = False
+            header = _("ERROR - The following hard dependencies are unmet:")
+            print(header)
+            print("=" * len(header))
+        print(dependency)
 
     for dependency in SOFT_DEPENDENCIES:
         dependency.check()
@@ -229,12 +288,29 @@ def check_requirements():
 
     if not _check_audiosinks():
         print((_("Could not create audio output sink. "
-                "Make sure you have a valid one (pulsesink, alsasink or osssink).")))
+                 "Make sure you have a valid one (pulsesink, alsasink or osssink).")))
+        return False
+
+    if not _check_videosink():
+        print((_("Could not create video output sink. "
+                 "Make sure you have a gtksink available.")))
         return False
 
     return True
 
 
+def require_version(modulename, version):
+    import gi
+
+    try:
+        gi.require_version(modulename, version)
+    except ValueError:
+        print((_("Could not import '%s'. "
+                 "Make sure you have it available."
+                 % modulename)))
+        exit(1)
+
+
 def initialize_modules():
     """
     Initialize the modules.
@@ -242,20 +318,41 @@ def initialize_modules():
     This has to be done in a specific order otherwise the app
     crashes on some systems.
     """
+    try:
+        import gi
+    except ImportError:
+        print((_("Could not import 'gi'. "
+                 "Make sure you have pygobject available.")))
+        exit(1)
+
+    require_version("Gtk", GTK_API_VERSION)
+    require_version("Gdk", GTK_API_VERSION)
     from gi.repository import Gdk
     Gdk.init([])
-    from gi.repository import GtkClutter
-    GtkClutter.init([])
 
-    import gi
     if not gi.version_info >= (3, 11):
         from gi.repository import GObject
         GObject.threads_init()
 
+    require_version("Gst", GST_API_VERSION)
+    require_version("GstController", GST_API_VERSION)
     from gi.repository import Gst
     Gst.init(None)
+
+    require_version("GES", GST_API_VERSION)
     from gi.repository import GES
-    GES.init()
+    res, sys.argv = GES.init_check(sys.argv)
+
+    from pitivi.utils import validate
+    if validate.init() and "--inspect-action-type" in sys.argv:
+        try:
+            action_type = [sys.argv[1 + sys.argv.index("--inspect-action-type")]]
+        except IndexError:
+            action_type = []
+        if validate.GstValidate.print_action_types(action_type):
+            exit(0)
+        else:
+            exit(1)
 
 
 """
@@ -273,25 +370,27 @@ Those are either:
 Some of our dependencies have version numbers requirements; for those without
 a specific version requirement, they have the "None" value.
 """
-HARD_DEPENDENCIES = [CairoDependency("1.10.0"),
-                     GtkOrClutterDependency("Clutter", "1.12.0"),
-                     GstDependency("Gst", "1.4.0"),
-                     GstDependency("GES", "1.4.0.0"),
-                     GtkOrClutterDependency("Gtk", "3.10.0"),
-                     ClassicDependency("numpy", None),
-                     GIDependency("Gio", None),
+GST_API_VERSION = "1.0"
+GTK_API_VERSION = "3.0"
+GLIB_API_VERSION = "2.0"
+HARD_DEPENDENCIES = [GICheck("3.14.0"),
+                     CairoDependency("1.10.0"),
+                     GstDependency("Gst", GST_API_VERSION, "1.6.0"),
+                     GstDependency("GES", GST_API_VERSION, "1.6.0.0"),
+                     GtkDependency("Gtk", GTK_API_VERSION, "3.10.0"),
+                     ClassicDependency("numpy"),
+                     GIDependency("Gio", "2.0"),
+                     GstPluginDependency("gstgtk", "1.6.0"),
+                     ClassicDependency("matplotlib"),
                      ]
 
-# This one is a special case: eventually gnonlin will be dropped
-ges_1_5 = GstDependency("GES", "1.5.0.0")
-ges_1_5.check()
-if not ges_1_5.satisfied:
-    HARD_DEPENDENCIES.append(GstPluginDependency("gnonlin", "1.4.0"))
-
 SOFT_DEPENDENCIES = \
     (
         ClassicDependency("pycanberra", None, _("enables sound notifications when rendering is complete")),
-        GIDependency("GnomeDesktop", None, _("file thumbnails provided by GNOME's thumbnailers")),
-        GIDependency("Notify", None, _("enables visual notifications when rendering is complete")),
+        GIDependency("GnomeDesktop", 1.0, None, _("file thumbnails provided by GNOME's thumbnailers")),
+        GIDependency("Notify", None, 1.0, _("enables visual notifications when rendering is complete")),
         GstPluginDependency("libav", None, _("additional multimedia codecs through the GStreamer Libav library")),
+        GstPluginDependency("debugutilsbad", None, _("enables a watchdog in the GStreamer pipeline."
+                                                     " Use to detect errors happening in GStreamer"
+                                                     " and recover from them")),
     )
