@@ -132,8 +132,8 @@ BLACKLISTED_EFFECTS = ["colorconvert", "coglogoinsert", "festival",
                        "alphacolor", "cogcolorspace", "videodetect",
                        "navigationtest", "videoanalyse"]
 
-#FIXME Check if this is still true with GES
-#We should unblacklist it when #650985 is solved
+# FIXME Check if this is still true with GES
+# We should unblacklist it when #650985 is solved
 BLACKLISTED_PLUGINS = ["ldaspa"]
 
 ICON_WIDTH = 48 + 2 * 6  # 48 pixels, plus a margin on each side
@@ -144,7 +144,7 @@ class EffectFactory(object):
     Factories that applies an effect on a stream
     """
     def __init__(self, effect_name, media_type, categories,
-                human_name, description):
+                 human_name, description):
         object.__init__(self)
         self.effect_name = effect_name
         self.media_type = media_type
@@ -154,6 +154,7 @@ class EffectFactory(object):
 
 
 class EffectsManager(object):
+
     """
     Groups effects.
     """
@@ -176,8 +177,8 @@ class EffectsManager(object):
             klass = element_factory.get_klass()
             name = element_factory.get_name()
 
-            if ("Effect" in klass and name not in BLACKLISTED_EFFECTS
-            and not [bplug for bplug in BLACKLISTED_PLUGINS if bplug in name]):
+            if ("Effect" in klass and name not in BLACKLISTED_EFFECTS and not
+                    [bplug for bplug in BLACKLISTED_PLUGINS if bplug in name]):
                 media_type = None
 
                 if "Audio" in klass:
@@ -193,8 +194,10 @@ class EffectsManager(object):
 
                 effect = EffectFactory(name,
                                        media_type,
-                                       categories=self._getEffectCategories(name),
-                                       human_name=self._getEffectName(element_factory),
+                                       categories=self._getEffectCategories(
+                                           name),
+                                       human_name=self._getEffectName(
+                                           element_factory),
                                        description=self._getEffectDescripton(element_factory))
                 self._addEffectToDic(name, effect)
 
@@ -300,7 +303,7 @@ class EffectsManager(object):
         return icon
 
 
-#----------------------- UI classes to manage effects -------------------------#
+# ----------------------- UI classes to manage effects -------------------------#
 HIDDEN_EFFECTS = ["frei0r-filter-scale0tilt"]
 
 GlobalSettings.addConfigSection('effect-library')
@@ -314,11 +317,12 @@ GlobalSettings.addConfigSection('effect-library')
  COL_ICON) = list(range(7))
 
 
-class EffectListWidget(Gtk.VBox, Loggable):
+class EffectListWidget(Gtk.Box, Loggable):
+
     """ Widget for listing effects """
 
     def __init__(self, instance, unused_uiman):
-        Gtk.VBox.__init__(self)
+        Gtk.Box.__init__(self)
         Loggable.__init__(self)
 
         self.app = instance
@@ -326,6 +330,7 @@ class EffectListWidget(Gtk.VBox, Loggable):
         self._draggedItems = None
         self._effectType = VIDEO_EFFECT
 
+        self.set_orientation(Gtk.Orientation.VERTICAL)
         builder = Gtk.Builder()
         builder.add_from_file(os.path.join(get_ui_dir(), "effectslibrary.ui"))
         builder.connect_signals(self)
@@ -337,7 +342,8 @@ class EffectListWidget(Gtk.VBox, Loggable):
         self.searchEntry = builder.get_object("search_entry")
 
         # Store
-        self.storemodel = Gtk.ListStore(str, str, int, object, object, str, GdkPixbuf.Pixbuf)
+        self.storemodel = Gtk.ListStore(
+            str, str, int, object, object, str, GdkPixbuf.Pixbuf)
 
         self.view = Gtk.TreeView(model=self.storemodel)
         self.view.props.headers_visible = False
@@ -349,7 +355,7 @@ class EffectListWidget(Gtk.VBox, Loggable):
         self.view.set_model(self.modelFilter)
 
         icon_col = Gtk.TreeViewColumn()
-        icon_col.props.spacing = SPACING
+        icon_col.set_spacing(SPACING)
         icon_col.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
         icon_col.props.fixed_width = ICON_WIDTH
         icon_cell = Gtk.CellRendererPixbuf()
@@ -366,13 +372,15 @@ class EffectListWidget(Gtk.VBox, Loggable):
         text_cell.props.xpad = 6
         text_cell.set_property("ellipsize", Pango.EllipsizeMode.END)
         text_col.pack_start(text_cell, True)
-        text_col.set_cell_data_func(text_cell, self.view_description_cell_data_func, None)
+        text_col.set_cell_data_func(
+            text_cell, self.view_description_cell_data_func, None)
 
         self.view.append_column(icon_col)
         self.view.append_column(text_col)
 
         # Make the treeview a drag source which provides effects.
-        self.view.enable_model_drag_source(Gdk.ModifierType.BUTTON1_MASK, [EFFECT_TARGET_ENTRY], Gdk.DragAction.COPY)
+        self.view.enable_model_drag_source(
+            Gdk.ModifierType.BUTTON1_MASK, [EFFECT_TARGET_ENTRY], Gdk.DragAction.COPY)
 
         self.view.connect("button-press-event", self._buttonPressEventCb)
         self.view.connect("select-cursor-row", self._enterPressEventCb)
@@ -420,7 +428,8 @@ class EffectListWidget(Gtk.VBox, Loggable):
                                         effect_factory,
                                         name,
                                         self.app.effects.getEffectIcon(name)])
-        self.storemodel.set_sort_column_id(COL_NAME_TEXT, Gtk.SortType.ASCENDING)
+        self.storemodel.set_sort_column_id(
+            COL_NAME_TEXT, Gtk.SortType.ASCENDING)
 
     def populate_categories_widget(self):
         self.categoriesWidget.get_model().clear()
@@ -454,7 +463,8 @@ class EffectListWidget(Gtk.VBox, Loggable):
     def _enterPressEventCb(self, unused_view, unused_event=None):
         factory_name = self.getSelectedEffectFactoryName()
         if factory_name is not None:
-            self.app.gui.clipconfig.effect_expander.addEffectToCurrentSelection(factory_name)
+            self.app.gui.clipconfig.effect_expander.addEffectToCurrentSelection(
+                factory_name)
 
     def _buttonPressEventCb(self, view, event):
         chain_up = True
@@ -464,7 +474,8 @@ class EffectListWidget(Gtk.VBox, Loggable):
         elif event.type == getattr(Gdk.EventType, '2BUTTON_PRESS'):
             factory_name = self.getSelectedEffectFactoryName()
             if factory_name is not None:
-                self.app.gui.clipconfig.effect_expander.addEffectToCurrentSelection(factory_name)
+                self.app.gui.clipconfig.effect_expander.addEffectToCurrentSelection(
+                    factory_name)
         else:
             chain_up = not self._rowUnderMouseSelected(view, event)
 
@@ -529,9 +540,11 @@ PROPS_TO_IGNORE = ['name', 'qos', 'silent', 'message']
 
 
 class EffectsPropertiesManager:
+
     """
     @type app: L{Pitivi}
     """
+
     def __init__(self, app):
         self.cache_dict = {}
         self._current_effect_setting_ui = None
@@ -549,11 +562,11 @@ class EffectsPropertiesManager:
             # Here we should handle special effects configuration UI
             effect_settings_widget = GstElementSettingsWidget()
             effect_settings_widget.setElement(effect, ignore=PROPS_TO_IGNORE,
-                    default_btn=True, use_element_props=True)
+                                              default_btn=True, use_element_props=True)
             scrolled_window = Gtk.ScrolledWindow()
             scrolled_window.add_with_viewport(effect_settings_widget)
             scrolled_window.set_policy(Gtk.PolicyType.AUTOMATIC,
-                    Gtk.PolicyType.AUTOMATIC)
+                                       Gtk.PolicyType.AUTOMATIC)
             self.cache_dict[effect] = scrolled_window
             self._connectAllWidgetCallbacks(effect_settings_widget, effect)
             self._postConfiguration(effect, effect_settings_widget)
@@ -561,7 +574,8 @@ class EffectsPropertiesManager:
         self._current_effect_setting_ui = self._getUiToSetEffect(effect)
         element = self._current_effect_setting_ui.element
         for prop in element.list_children_properties():
-            self._current_element_values[prop.name] = element.get_child_property(prop.name)
+            self._current_element_values[
+                prop.name] = element.get_child_property(prop.name)
 
         return self.cache_dict[effect]
 
@@ -580,7 +594,8 @@ class EffectsPropertiesManager:
     def _getUiToSetEffect(self, effect):
         """ Permit to get the widget to set the effect and not its container """
         if type(self.cache_dict[effect]) is Gtk.ScrolledWindow:
-            effect_set_ui = self.cache_dict[effect].get_children()[0].get_children()[0]
+            effect_set_ui = self.cache_dict[
+                effect].get_children()[0].get_children()[0]
         else:
             effect_set_ui = self.cache_dict[effect]
         return effect_set_ui
@@ -595,13 +610,14 @@ class EffectsPropertiesManager:
     def _onValueChangedCb(self, unused_widget, dynamic, prop):
         value = dynamic.getWidgetValue()
 
-        #FIXME Workaround in order to make aspectratiocrop working
+        # FIXME Workaround in order to make aspectratiocrop working
         if isinstance(value, Gst.Fraction):
             value = Gst.Fraction(int(value.num), int(value.denom))
 
         if value != self._current_element_values.get(prop.name):
             self.action_log.begin("Effect property change")
-            self._current_effect_setting_ui.element.set_child_property(prop.name, value)
+            self._current_effect_setting_ui.element.set_child_property(
+                prop.name, value)
             self.action_log.commit()
 
             self.app.project_manager.current_project.pipeline.flushSeek()
