@@ -35,11 +35,13 @@ from pitivi.utils import system
 
 
 class DuplicatePresetNameException(Exception):
+
     """Raised when an operation would result in a duplicated preset name."""
     pass
 
 
 class PresetManager(object):
+
     """Abstract class for storing a list of presets.
 
     Subclasses must provide a filename attribute.
@@ -65,7 +67,7 @@ class PresetManager(object):
         self.ordered = Gtk.ListStore(str, object)
         self.cur_preset = None
         # Whether to ignore the updateValue calls.
-        self._ignore_update_requests = False
+        self.ignore_update_requests = False
         self.system = system.getSystem()
 
     def loadAll(self):
@@ -178,7 +180,7 @@ class PresetManager(object):
 
     def updateValue(self, name, value):
         """Update a value in the current preset, if any."""
-        if self._ignore_update_requests:
+        if self.ignore_update_requests:
             # This is caused by restorePreset, nothing to do.
             return
         if self.cur_preset:
@@ -194,20 +196,22 @@ class PresetManager(object):
         @param preset: The name of the preset to be selected.
         @type preset: str
         """
-        self._ignore_update_requests = True
         if preset is None:
             self.cur_preset = None
             return
-        elif preset not in self.presets:
+        if preset not in self.presets:
             return
-        values = self.presets[preset]
-        self.cur_preset = preset
-        for field, (setter, getter) in self.widget_map.items():
-            if values[field] != 0:
-                setter(values[field])
-            else:
-                setter(self.presets[_("No preset")][field])
-        self._ignore_update_requests = False
+        self.ignore_update_requests = True
+        try:
+            values = self.presets[preset]
+            self.cur_preset = preset
+            for field, (setter, getter) in self.widget_map.items():
+                if values[field] != 0:
+                    setter(values[field])
+                else:
+                    setter(self.presets[_("No preset")][field])
+        finally:
+            self.ignore_update_requests = False
 
     def saveCurrentPreset(self):
         """Update the current preset values from the widgets and save it."""
@@ -232,7 +236,8 @@ class PresetManager(object):
 
     def removePreset(self, name):
         try:
-            os.remove(self.presets[name]["filepath"])  # Deletes json file if exists
+            # Deletes json file if exists
+            os.remove(self.presets[name]["filepath"])
         except KeyError:
             # Trying to remove a preset that has not actually been saved
             return
@@ -302,7 +307,8 @@ class VideoPresetManager(PresetManager):
     user_path = os.path.join(xdg_data_home(), 'video_presets')
 
     def _loadSection(self, filepath):
-        parser = json.loads(open(filepath).read())
+        with open(filepath) as section:
+            parser = json.loads(section.read())
 
         name = parser["name"]
         width = parser["width"]
@@ -344,10 +350,10 @@ class AudioPresetManager(PresetManager):
     user_path = os.path.join(xdg_data_home(), 'audio_presets')
 
     def _loadSection(self, filepath):
-        parser = json.loads(open(filepath).read())
+        with open(filepath) as section:
+            parser = json.loads(section.read())
 
         name = parser["name"]
-
         channels = parser["channels"]
         sample_rate = parser["sample-rate"]
 
@@ -373,7 +379,8 @@ class RenderPresetManager(PresetManager):
     user_path = os.path.join(xdg_data_home(), 'render_presets')
 
     def _loadSection(self, filepath):
-        parser = json.loads(open(filepath).read())
+        with open(filepath) as section:
+            parser = json.loads(section.read())
 
         name = parser["name"]
         container = parser["container"]
@@ -381,9 +388,9 @@ class RenderPresetManager(PresetManager):
         vcodec = parser["vcodec"]
 
         cached_encs = CachedEncoderList()
-        if (acodec not in [fact.get_name() for fact in cached_encs.aencoders]
-        or vcodec not in [fact.get_name() for fact in cached_encs.vencoders]
-        or container not in [fact.get_name() for fact in cached_encs.muxers]):
+        if (acodec not in [fact.get_name() for fact in cached_encs.aencoders] or
+                vcodec not in [fact.get_name() for fact in cached_encs.vencoders] or
+                container not in [fact.get_name() for fact in cached_encs.muxers]):
             return
 
         try:
