@@ -20,11 +20,12 @@
 # Boston, MA 02110-1301, USA.
 
 import os
-from configparser import SafeConfigParser, ParsingError
+import configparser
 
 from gi.repository import GLib
 from gi.repository import GObject
 
+from pitivi.utils.loggable import Loggable
 from pitivi.utils.misc import unicode_error_dialog
 
 
@@ -99,6 +100,7 @@ class ConfigError(Exception):
 
 
 class Notification(object):
+
     """A descriptor to help with the implementation of signals"""
 
     def __init__(self, attrname):
@@ -117,7 +119,8 @@ class Notification(object):
         instance.emit(self.signame)
 
 
-class GlobalSettings(GObject.Object):
+class GlobalSettings(GObject.Object, Loggable):
+
     """
     Pitivi app settings.
 
@@ -140,7 +143,10 @@ class GlobalSettings(GObject.Object):
 
     def __init__(self, **unused_kwargs):
         GObject.Object.__init__(self)
-        self._config = SafeConfigParser()
+        Loggable.__init__(self)
+
+        self.conf_file_path = os.path.join(xdg_config_home(), "pitivi.conf")
+        self._config = configparser.ConfigParser()
         self._readSettingsFromConfigurationFile()
         self._readSettingsFromEnvironmentVariables()
 
@@ -150,12 +156,13 @@ class GlobalSettings(GObject.Object):
         """
 
         try:
-            conf_file_path = os.path.join(xdg_config_home(), "pitivi.conf")
-            self._config.read(conf_file_path)
-        except UnicodeDecodeError:
+            self._config.read(self.conf_file_path)
+        except UnicodeDecodeError as e:
+            self.error("Failed to read %s: %s", self.conf_file_path, e)
             unicode_error_dialog()
             return
-        except ParsingError:
+        except configparser.ParsingError as e:
+            self.error("Failed to parse %s: %s", self.conf_file_path, e)
             return
 
         for (section, attrname, typ, key, env, value) in self.iterAllOptions():
@@ -217,8 +224,6 @@ class GlobalSettings(GObject.Object):
                 setattr(self, attrname, var)
 
     def _writeSettingsToConfigurationFile(self):
-        conf_file_path = os.path.join(xdg_config_home(), "pitivi.conf")
-
         for (section, attrname, typ, key, env_var, value) in self.iterAllOptions():
             if not self._config.has_section(section):
                 self._config.add_section(section)
@@ -228,11 +233,10 @@ class GlobalSettings(GObject.Object):
                 else:
                     self._config.remove_option(section, key)
         try:
-            file = open(conf_file_path, 'w')
-        except IOError as OSError:
-            return
-        self._config.write(file)
-        file.close()
+            with open(self.conf_file_path, 'w') as file:
+                self._config.write(file)
+        except (IOError, OSError) as e:
+            self.error("Failed to write to %s: %s", self.conf_file_path, e)
 
     def storeSettings(self):
         """
@@ -245,9 +249,6 @@ class GlobalSettings(GObject.Object):
     def iterAllOptions(self):
         """
         Iterate over all registered options
-
-        @return: an iterator which yields a tuple of (attrname, type, key,
-        environment, value) for each option.
         """
         for section, options in list(self.options.items()):
             for attrname, (typ, key, environment) in list(options.items()):
@@ -289,19 +290,21 @@ class GlobalSettings(GObject.Object):
         @type notify: C{boolean}
         """
         if section and section not in cls.options:
-            raise ConfigError("You must add the section \"%s\" first." % section)
+            raise ConfigError(
+                "You must add the section \"%s\" first." % section)
         if key and not section:
-            raise ConfigError("You must specify a section for key \"%s\"" % key)
+            raise ConfigError(
+                "You must specify a section for key \"%s\"" % key)
         if section and key in cls.options[section]:
             raise ConfigError("Option \"%s\" is already in use.")
         if hasattr(cls, attrname):
             raise ConfigError("Settings attribute \"%s\" is already in use.")
         if environment and environment in cls.environment:
             raise ConfigError("Settings environment varaible \"%s\" is"
-                "already in use.")
+                              "already in use.")
         if not type_ and default is None:
             raise ConfigError("Settings attribute \"%s\" has must have a"
-                " type or a default." % attrname)
+                              " type or a default." % attrname)
         if not type_:
             type_ = type(default)
         if notify:
@@ -334,5 +337,8 @@ class GlobalSettings(GObject.Object):
 
     @classmethod
     def notifiesConfigOption(cls, attrname):
+        """
+        Whether a changed signal is emitted for the specified setting.
+        """
         signal_name = Notification.signalName(attrname)
-        GObject.signal_lookup(signal_name, cls)
+        return bool(GObject.signal_lookup(signal_name, cls))
