@@ -34,7 +34,10 @@ from gettext import gettext as _
 from pitivi.utils.pipeline import Seeker
 from pitivi.utils.timeline import Zoomable
 from pitivi.utils.loggable import Loggable
-from pitivi.utils.ui import NORMAL_FONT, PLAYHEAD_COLOR, PLAYHEAD_WIDTH, set_cairo_color, time_to_string, beautify_length
+from pitivi.utils.ui import NORMAL_FONT, PLAYHEAD_WIDTH, set_cairo_color, time_to_string, beautify_length
+
+
+HEIGHT = 25
 
 # A series of valid interval lengths in seconds.
 SCALES = (0.5, 1, 2, 5, 10, 15, 30, 60, 120, 300, 600, 3600)
@@ -57,6 +60,7 @@ SMALL_FONT_SIZE = 11
 
 
 class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
+
     """
     Widget for displaying the ruler.
 
@@ -70,7 +74,7 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         "motion-notify-event": "override",
         "scroll-event": "override",
         "seek": (GObject.SignalFlags.RUN_LAST, None,
-                [GObject.TYPE_UINT64])
+                 [GObject.TYPE_UINT64])
     }
 
     def __init__(self, timeline, hadj):
@@ -85,13 +89,14 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         self.connect("focus-out-event", self._focusOutCb)
 
         self.timeline = timeline
-        self._background_color = timeline.get_style_context().lookup_color('theme_bg_color')[1]
+        self._background_color = timeline.get_style_context().lookup_color(
+            'theme_bg_color')[1]
         self._seeker = Seeker()
         self.hadj = hadj
         hadj.connect("value-changed", self._hadjValueChangedCb)
         self.add_events(Gdk.EventMask.POINTER_MOTION_MASK |
-            Gdk.EventMask.BUTTON_PRESS_MASK | Gdk.EventMask.BUTTON_RELEASE_MASK |
-            Gdk.EventMask.SCROLL_MASK)
+                        Gdk.EventMask.BUTTON_PRESS_MASK | Gdk.EventMask.BUTTON_RELEASE_MASK |
+                        Gdk.EventMask.SCROLL_MASK)
 
         self.pixbuf = None
 
@@ -108,8 +113,7 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         self.connect('draw', self.drawCb)
         self.connect('configure-event', self.configureEventCb)
         self.callback_id = None
-        self.callback_id_scroll = None
-        self.set_size_request(0, 25)
+        self.set_size_request(0, HEIGHT)
 
         style = self.get_style_context()
         color_normal = style.get_color(Gtk.StateFlags.NORMAL)
@@ -133,22 +137,12 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
 
     def _hadjValueChangedCb(self, unused_arg):
         self.pixbuf_offset = self.hadj.get_value()
-        if self.callback_id_scroll is not None:
-            GLib.source_remove(self.callback_id_scroll)
-        self.callback_id_scroll = GLib.timeout_add(100, self._maybeUpdate)
+        self.queue_draw()
 
 # Zoomable interface override
 
-    def _maybeUpdate(self):
-        self.queue_draw()
-        self.callback_id = None
-        self.callback_id_scroll = None
-        return False
-
     def zoomChanged(self):
-        if self.callback_id is not None:
-            GLib.source_remove(self.callback_id)
-        self.callback_id = GLib.timeout_add(100, self._maybeUpdate)
+        self.queue_draw()
 
 # Timeline position changed method
 
@@ -223,17 +217,15 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
             # Control + scroll = zoom
             if event.scroll.direction == Gdk.ScrollDirection.UP:
                 Zoomable.zoomIn()
-                self.timeline.zoomed_fitted = False
             elif event.scroll.direction == Gdk.ScrollDirection.DOWN:
                 Zoomable.zoomOut()
-                self.timeline.zoomed_fitted = False
         else:
             # No modifier key held down, just scroll
-            if (event.scroll.direction == Gdk.ScrollDirection.UP
-            or event.scroll.direction == Gdk.ScrollDirection.LEFT):
+            if event.scroll.direction in (Gdk.ScrollDirection.UP,
+                                          Gdk.ScrollDirection.LEFT):
                 self.timeline.scroll_left()
-            elif (event.scroll.direction == Gdk.ScrollDirection.DOWN
-            or event.scroll.direction == Gdk.ScrollDirection.RIGHT):
+            elif event.scroll.direction in (Gdk.ScrollDirection.DOWN,
+                                            Gdk.ScrollDirection.RIGHT):
                 self.timeline.scroll_right()
 
     def setProjectFrameRate(self, rate):
@@ -242,7 +234,8 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         """
         self.frame_rate = rate
         self.ns_per_frame = float(1 / self.frame_rate) * Gst.SECOND
-        self.scales = (float(2 / rate), float(5 / rate), float(10 / rate)) + SCALES
+        self.scales = (float(2 / rate), float(
+            5 / rate), float(10 / rate)) + SCALES
 
 # Drawing methods
 
@@ -255,8 +248,10 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         context.fill()
         offset = int(self.nsToPixel(Gst.CLOCK_TIME_NONE)) - self.pixbuf_offset
         if offset > 0:
-            set_cairo_color(context, style.get_background_color(Gtk.StateFlags.ACTIVE))
-            context.rectangle(0, 0, int(offset), context.get_target().get_height())
+            set_cairo_color(
+                context, style.get_background_color(Gtk.StateFlags.ACTIVE))
+            context.rectangle(
+                0, 0, int(offset), height)
             context.fill()
 
     def drawRuler(self, context):
@@ -276,7 +271,9 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
             spacing = scale * zoom
             if spacing >= textwidth * 1.5:
                 return spacing, scale
-        raise Exception("Failed to find an interval size for textwidth:%s, zoomratio:%s" % (textwidth, Zoomable.zoomratio))
+        raise Exception(
+            "Failed to find an interval size for textwidth:%s, zoomratio:%s" %
+            (textwidth, Zoomable.zoomratio))
 
     def drawTicks(self, context, offset, spacing):
         for count_per_interval, height_ratio in TICK_TYPES:
@@ -370,13 +367,16 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         states = [style.get_background_color(Gtk.StateFlags.ACTIVE),
                   style.get_background_color(Gtk.StateFlags.SELECTED)]
 
-        frame_num = int(self.pixelToNs(self.pixbuf_offset) * float(self.frame_rate) / Gst.SECOND)
+        frame_num = int(
+            self.pixelToNs(self.pixbuf_offset) * float(self.frame_rate) / Gst.SECOND)
         paintpos = self.pixbuf_offset - offset
         max_pos = context.get_target().get_width() + self.pixbuf_offset
         while paintpos < max_pos:
-            paintpos = self.nsToPixel(1 / float(self.frame_rate) * Gst.SECOND * frame_num)
+            paintpos = self.nsToPixel(
+                1 / float(self.frame_rate) * Gst.SECOND * frame_num)
             set_cairo_color(context, states[(frame_num + 1) % 2])
-            context.rectangle(0.5 + paintpos - self.pixbuf_offset, y, frame_width, height)
+            context.rectangle(
+                0.5 + paintpos - self.pixbuf_offset, y, frame_width, height)
             context.fill()
             frame_num += 1
 
@@ -385,7 +385,7 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         # without this the line appears blurry.
         xpos = self.nsToPixel(self.position) - self.pixbuf_offset + 0.5
         context.set_line_width(PLAYHEAD_WIDTH + 2)
-        set_cairo_color(context, PLAYHEAD_COLOR)
+        set_cairo_color(context, (255, 0, 0))
         context.move_to(xpos, 0)
         context.line_to(xpos, context.get_target().get_height())
         context.stroke()
