@@ -24,8 +24,7 @@
 """
 High-level pipelines
 """
-import platform
-
+import os
 
 from gi.repository import GLib
 from gi.repository import GObject
@@ -34,24 +33,30 @@ from gi.repository import GES
 
 from pitivi.utils.loggable import Loggable
 from pitivi.utils.misc import format_ns
+from pitivi.check import videosink_factory
 
 
-MAX_RECOVERIES = 5
-
 PIPELINE_SIGNALS = {
-    "state-change": (GObject.SignalFlags.RUN_LAST, None, (GObject.TYPE_INT,)),
+    "state-change": (GObject.SignalFlags.RUN_LAST, None, (GObject.TYPE_INT, GObject.TYPE_INT)),
     "position": (GObject.SignalFlags.RUN_LAST, None, (GObject.TYPE_UINT64,)),
     "duration-changed": (GObject.SignalFlags.RUN_LAST, None, (GObject.TYPE_UINT64,)),
     "eos": (GObject.SignalFlags.RUN_LAST, None, ()),
     "error": (GObject.SignalFlags.RUN_LAST, None, (GObject.TYPE_STRING, GObject.TYPE_STRING)),
+    "died": (GObject.SignalFlags.RUN_LAST, None, ()),
 }
 
+MAX_RECOVERIES = 3
+WATCHDOG_TIMEOUT = 3
+MAX_BRINGING_TO_PAUSED_DURATION = 5
+MAX_SET_STATE_DURATION = 1
+
 
 class PipelineError(Exception):
     pass
 
 
 class Seeker(GObject.Object, Loggable):
+
     """
     The Seeker is a singleton helper class to do various seeking
     operations in the pipeline.
@@ -91,7 +96,8 @@ class Seeker(GObject.Object, Loggable):
 
         if self.pending_seek_id is None:
             if on_idle:
-                self.pending_seek_id = self._scheduleSeek(self.timeout, self._seekTimeoutCb)
+                self.pending_seek_id = self._scheduleSeek(
+                    self.timeout, self._seekTimeoutCb)
             else:
                 self._seekTimeoutCb()
         else:
@@ -101,7 +107,8 @@ class Seeker(GObject.Object, Loggable):
         if self.pending_seek_id is None:
             self._time = int(time)
             if on_idle:
-                self.pending_seek_id = self._scheduleSeek(self.timeout, self._seekTimeoutCb, relative=True)
+                self.pending_seek_id = self._scheduleSeek(
+                    self.timeout, self._seekTimeoutCb, relative=True)
             else:
                 self._seekTimeoutCb(relative=True)
 
@@ -144,6 +151,7 @@ class Seeker(GObject.Object, Loggable):
 
 
 class SimplePipeline(GObject.Object, Loggable):
+
     """
     The Pipeline is only responsible for:
      - State changes
@@ -160,6 +168,11 @@ class SimplePipeline(GObject.Object, Loggable):
 
     __gsignals__ = PIPELINE_SIGNALS
 
+    class RecoveryState(object):
+        NOT_RECOVERING = "not-recovering"
+        STARTED_RECOVERING = "started-recovering"
+        SEEKED_AFTER_RECOVERING = "seeked-after-recovering"
+
     def __init__(self, pipeline):
         GObject.Object.__init__(self)
         Loggable.__init__(self)
@@ -169,24 +182,39 @@ class SimplePipeline(GObject.Object, Loggable):
         self._bus.add_signal_watch()
         self._bus.connect("message", self._busMessageCb)
         self._listening = False  # for the position handler
-        self._listeningInterval = 300  # default 300ms
+        self._listeningInterval = 50  # default 300ms
         self._listeningSigId = 0
         self._duration = Gst.CLOCK_TIME_NONE
-        self.lastPosition = int(0 * Gst.SECOND)
-        self.pendingRecovery = False
+        self._last_position = int(0 * Gst.SECOND)
+        self._recovery_state = self.RecoveryState.NOT_RECOVERING
         self._attempted_recoveries = 0
-        self._waiting_for_async_done = True
+        self._waiting_for_async_done = False
         self._next_seek = None
         self._timeout_async_id = 0
+        self._force_position_listener = False
 
-        # Create a cluttersink element used for display. Subclasses must connect
-        # it to self._pipeline themselves
-        self.video_sink = Gst.ElementFactory.make("glimagesink", None)
-        if isinstance(pipeline, GES.Pipeline):
+        self.setSink(self.createSink())
+
+    def createSink(self):
+        sink = Gst.ElementFactory.make(videosink_factory.get_name(), None)
+        if videosink_factory.get_name() == "gtksink":
+            self.info("Using gtksink")
+            return sink
+
+        sinkbin = Gst.ElementFactory.make("glsinkbin", None)
+        sinkbin.props.sink = sink
+        return sinkbin
+
+    def setSink(self, sink):
+        self.video_sink = sink
+        if isinstance(self._pipeline, GES.Pipeline):
             self._pipeline.preview_set_video_sink(self.video_sink)
         else:
             self._pipeline.set_property("video_sink", self.video_sink)
 
+    def setForcePositionListener(self, force):
+        self._force_position_listener = force
+
     def release(self):
         """
         Release the L{Pipeline} and all used L{ObjectFactory} and
@@ -201,11 +229,14 @@ class SimplePipeline(GObject.Object, Loggable):
         self._bus.disconnect_by_func(self._busMessageCb)
         self._bus.remove_signal_watch()
 
-        self._pipeline.setState(Gst.State.NULL)
+        self._pipeline.set_state(Gst.State.NULL)
         self._bus = None
 
     def flushSeek(self):
-        self.pause()
+        if self.getState() == Gst.State.PLAYING:
+            self.debug("Playing, no need to flush here!")
+            return
+
         try:
             self.seekRelative(0)
         except PipelineError as e:
@@ -220,11 +251,24 @@ class SimplePipeline(GObject.Object, Loggable):
         the requested state.
         """
         self.debug("state set to: %r", state)
+        if state >= Gst.State.PAUSED:
+            cstate = self.getState()
+            if cstate < Gst.State.PAUSED:
+                if cstate == Gst.State.NULL:
+                    timeout = MAX_BRINGING_TO_PAUSED_DURATION
+                else:
+                    timeout = MAX_SET_STATE_DURATION
+
+                self._addWaitingForAsyncDoneTimeout(timeout)
+        else:
+            self._removeWaitingForAsyncDoneTimeout()
+
         res = self._pipeline.set_state(state)
         if res == Gst.StateChangeReturn.FAILURE:
             # reset to NULL
             self._pipeline.set_state(Gst.State.NULL)
-            raise PipelineError("Failure changing state of the Gst.Pipeline to %r, currently reset to NULL" % state)
+            raise PipelineError(
+                "Failure changing state of the Gst.Pipeline to %r, currently reset to NULL" % state)
 
     def getState(self):
         """
@@ -236,8 +280,10 @@ class SimplePipeline(GObject.Object, Loggable):
         @return: The current state.
         @rtype: C{State}
         """
-        change, state, pending = self._pipeline.get_state(timeout=0)  # No timeout
-        self.debug("change: %r, state: %r, pending: %r", change, state, pending)
+        # No timeout
+        change, state, pending = self._pipeline.get_state(timeout=0)
+        self.debug(
+            "change: %r, state: %r, pending: %r", change, state, pending)
         return state
 
     def play(self):
@@ -252,18 +298,6 @@ class SimplePipeline(GObject.Object, Loggable):
         """
         self.setState(Gst.State.PAUSED)
 
-        # When the pipeline has been paused we need to update the
-        # timeline/playhead position, as the 'position' signal
-        # is only emitted every 300ms and the playhead jumps
-        # during the playback.
-        try:
-            position = self.getPosition()
-        except PipelineError as e:
-            self.warning("Getting the position failed: %s", e)
-            return
-        if position != Gst.CLOCK_TIME_NONE and position >= 0:
-            self.emit("position", position)
-
     def stop(self):
         """
         Sets the L{Pipeline} to READY
@@ -278,7 +312,7 @@ class SimplePipeline(GObject.Object, Loggable):
 
     # Position and Seeking methods
 
-    def getPosition(self):
+    def getPosition(self, blocks=False):
         """
         Get the current position of the L{Pipeline}.
 
@@ -286,6 +320,12 @@ class SimplePipeline(GObject.Object, Loggable):
         @rtype: L{long}
         @raise PipelineError: If the position couldn't be obtained.
         """
+        maincontext = GLib.main_context_default()
+        if blocks and self._recovery_state == self.RecoveryState.NOT_RECOVERING:
+            while self._waiting_for_async_done and self._recovery_state == self.RecoveryState.NOT_RECOVERING:
+                self.info("Iterating mainloop waiting for the pipeline to be ready to be queried")
+                maincontext.iteration(True)
+
         try:
             res, cur = self._pipeline.query_position(Gst.Format.TIME)
         except Exception as e:
@@ -309,7 +349,7 @@ class SimplePipeline(GObject.Object, Loggable):
         self._duration = dur
         return dur
 
-    def activatePositionListener(self, interval=500):
+    def activatePositionListener(self, interval=50):
         """
         Activate the position listener.
 
@@ -326,7 +366,7 @@ class SimplePipeline(GObject.Object, Loggable):
             return True
         self._listening = True
         self._listeningInterval = interval
-        # if we're in paused or playing, switch it on
+        # if we're in playing, switch it on
         self._listenToPosition(self.getState() == Gst.State.PLAYING)
         return True
 
@@ -348,7 +388,7 @@ class SimplePipeline(GObject.Object, Loggable):
             else:
                 if position != Gst.CLOCK_TIME_NONE:
                     self.emit('position', position)
-                    self.lastPosition = position
+                    self._last_position = position
         finally:
             return True
 
@@ -357,17 +397,32 @@ class SimplePipeline(GObject.Object, Loggable):
         # i.e. it does NOT check for current state
         if listen:
             if self._listening and self._listeningSigId == 0:
-                self._listeningSigId = GLib.timeout_add(self._listeningInterval,
+                self._listeningSigId = GLib.timeout_add(
+                    self._listeningInterval,
                     self._positionListenerCb)
         elif self._listeningSigId != 0:
             GLib.source_remove(self._listeningSigId)
             self._listeningSigId = 0
 
-    def _resetWaitingForAsyncDone(self):
-        self.warning("we didn't get async done, this is a bug")
-        self._waiting_for_async_done = False
+    def _asyncDoneNotReceivedCb(self):
+        self.error("we didn't get async done, this is a bug")
+        self._recover()
+        # Source is being removed
+        self._removeWaitingForAsyncDoneTimeout()
         return False
 
+    def _removeWaitingForAsyncDoneTimeout(self):
+        if self._timeout_async_id:
+            GLib.source_remove(self._timeout_async_id)
+        self._timeout_async_id = 0
+
+    def _addWaitingForAsyncDoneTimeout(self, timeout=WATCHDOG_TIMEOUT):
+        self._removeWaitingForAsyncDoneTimeout()
+
+        self._timeout_async_id = GLib.timeout_add_seconds(timeout,
+                                                          self._asyncDoneNotReceivedCb)
+        self._waiting_for_async_done = True
+
     def simple_seek(self, position):
         """
         Seeks in the L{Pipeline} to the given position.
@@ -384,21 +439,17 @@ class SimplePipeline(GObject.Object, Loggable):
         self.debug("position: %s", format_ns(position))
 
         # clamp between [0, duration]
-        position = max(0, min(position, self.getDuration()) - 1)
+        position = max(0, min(position, self.getDuration()))
 
-        res = self._pipeline.seek(1.0, Gst.Format.TIME, Gst.SeekFlags.FLUSH,
+        res = self._pipeline.seek(1.0, Gst.Format.TIME, Gst.SeekFlags.FLUSH | Gst.SeekFlags.ACCURATE,
                                   Gst.SeekType.SET, position,
                                   Gst.SeekType.NONE, -1)
-        self._waiting_for_async_done = True
-
-        if self._timeout_async_id:
-            GLib.source_remove(self._timeout_async_id)
-        self._timeout_async_id = GLib.timeout_add(1000, self._resetWaitingForAsyncDone)
+        self._addWaitingForAsyncDoneTimeout()
 
         if not res:
-            raise PipelineError("seek failed: %s" % position)
+            raise PipelineError(self.get_name() + " seek failed: " + str(position))
 
-        self.lastPosition = position
+        self._last_position = position
 
         self.debug("seeking successful")
         self.emit('position', position)
@@ -416,7 +467,8 @@ class SimplePipeline(GObject.Object, Loggable):
             prev, new, pending = message.parse_state_changed()
 
             if message.src == self._pipeline:
-                self.debug("Pipeline change state prev: %r, new: %r, pending: %r", prev, new, pending)
+                self.debug(
+                    "Pipeline change state prev: %r, new: %r, pending: %r", prev, new, pending)
 
                 emit_state_change = pending == Gst.State.VOID_PENDING
                 if prev == Gst.State.READY and new == Gst.State.PAUSED:
@@ -427,47 +479,89 @@ class SimplePipeline(GObject.Object, Loggable):
                         self.warning("Could not get duration because: %s", e)
                         # no sinks??
                         pass
-                    if self.pendingRecovery:
-                        self.simple_seek(self.lastPosition)
-                        self.pendingRecovery = False
-                        self.info("Seeked back to the last position after pipeline recovery")
+
+                    if self._recovery_state == self.RecoveryState.STARTED_RECOVERING:
+                        if self._attempted_recoveries == MAX_RECOVERIES:
+                            self._recovery_state = self.RecoveryState.NOT_RECOVERING
+                            self._attempted_recoveries = 0
+                            self.error("Too many tries to seek back to right position"
+                                       "not trying again, and going back to 0 instead")
+                        else:
+                            self._recovery_state = self.RecoveryState.SEEKED_AFTER_RECOVERING
+                            self.simple_seek(self._last_position)
+                            self.info(
+                                "Seeked back to the last position after pipeline recovery")
+                    self._listenToPosition(self._force_position_listener)
                 elif prev == Gst.State.PAUSED and new == Gst.State.PLAYING:
                     self._listenToPosition(True)
                 elif prev == Gst.State.PLAYING and new == Gst.State.PAUSED:
-                    self._listenToPosition(False)
+                    self._listenToPosition(self._force_position_listener)
 
                 if emit_state_change:
-                    self.emit('state-change', new)
+                    self.emit('state-change', new, prev)
 
         elif message.type == Gst.MessageType.ERROR:
             error, detail = message.parse_error()
             self._handleErrorMessage(error, detail, message.src)
-            self._waiting_for_async_done = False
-            if not (self._pipeline.get_mode() & GES.PipelineFlags.RENDER):
+            Gst.debug_bin_to_dot_file_with_ts(self._pipeline,
+                                              Gst.DebugGraphDetails.ALL,
+                                              "pitivi.error")
+            if not self._rendering():
                 self._recover()
         elif message.type == Gst.MessageType.DURATION_CHANGED:
             self.debug("Duration might have changed, querying it")
             GLib.idle_add(self._queryDurationAsync)
         elif message.type == Gst.MessageType.ASYNC_DONE:
-            self._attempted_recoveries = 0
+            if self._recovery_state == self.RecoveryState.SEEKED_AFTER_RECOVERING:
+                self._recovery_state = self.RecoveryState.NOT_RECOVERING
+                self._attempted_recoveries = 0
             self._waiting_for_async_done = False
+            self.__emitPosition()
             if self._next_seek is not None:
                 self.simple_seek(self._next_seek)
                 self._next_seek = None
-            if self._timeout_async_id:
-                GLib.source_remove(self._timeout_async_id)
-                self._timeout_async_id = 0
+            self._removeWaitingForAsyncDoneTimeout()
         else:
             self.log("%s [%r]", message.type, message.src)
 
+    def __emitPosition(self):
+        # When the pipeline has been paused we need to update the
+        # timeline/playhead position, as the 'position' signal
+        # is only emitted every 300ms and the playhead jumps
+        # during the playback.
+        try:
+            position = self.getPosition()
+        except PipelineError as e:
+            self.warning("Getting the position failed: %s", e)
+            return None
+
+        if position != Gst.CLOCK_TIME_NONE and position >= 0:
+            self.emit("position", position)
+
+        return position
+
+    @property
+    def _waiting_for_async_done(self):
+        return self.__waiting_for_async_done
+
+    @_waiting_for_async_done.setter
+    def _waiting_for_async_done(self, value):
+        self.__waiting_for_async_done = value
+
     def _recover(self):
         if self._attempted_recoveries > MAX_RECOVERIES:
-            self.warning("Pipeline error detected multiple times in a row, not resetting anymore")
+            self.emit("died")
+            self.error(
+                "Pipeline error detected multiple times in a row, not resetting anymore")
             return
-        self.warning("Pipeline error detected during playback, resetting")
-        self.pendingRecovery = True
-        self._pipeline.set_state(Gst.State.NULL)
-        self._pipeline.set_state(Gst.State.PAUSED)
+
+        self.error("Pipeline error detected during playback, resetting"
+                   " -- num tries: %d", self._attempted_recoveries)
+
+        self.setState(Gst.State.NULL)
+        self._recovery_state = self.RecoveryState.STARTED_RECOVERING
+        self.pause()
+
         self._attempted_recoveries += 1
 
     def _queryDurationAsync(self, *unused_args, **unused_kwargs):
@@ -492,8 +586,12 @@ class SimplePipeline(GObject.Object, Loggable):
             raise PipelineError("Couldn't get duration: Returned None")
         return dur
 
+    def _rendering(self):
+        return False
+
 
 class AssetPipeline(SimplePipeline):
+
     """
     Pipeline for playing a single clip.
     """
@@ -509,16 +607,9 @@ class AssetPipeline(SimplePipeline):
     def setClipUri(self, uri):
         self._pipeline.set_property("uri", uri)
 
-    def connectWithViewer(self, widget):
-        if platform.system() == 'Windows':
-            handle = widget.drawing_area.get_window().get_handle()
-        else:
-            handle = widget.drawing_area.get_window().get_xid()
-
-        self.video_sink.set_window_handle(handle)
-
 
 class Pipeline(GES.Pipeline, SimplePipeline):
+
     """
     Helper to handle GES.Pipeline through the SimplePipeline API
     and handle the Seeker properly
@@ -526,15 +617,33 @@ class Pipeline(GES.Pipeline, SimplePipeline):
 
     __gsignals__ = PIPELINE_SIGNALS
 
-    def __init__(self, pipeline=None):
+    def __init__(self, app, pipeline=None):
         GES.Pipeline.__init__(self)
         SimplePipeline.__init__(self, self)
 
+        self.app = app
+
+        self._was_empty = False
+        self._commit_wanted = False
+
         self._timeline = None
         self._seeker = Seeker()
         self._seeker.connect("seek", self._seekCb)
         self._seeker.connect("seek-relative", self._seekRelativeCb)
 
+        if "watchdog" in os.environ.get("PITIVI_UNSTABLE_FEATURES", ''):
+            watchdog = Gst.ElementFactory.make("watchdog", None)
+            if watchdog:
+                watchdog.props.timeout = WATCHDOG_TIMEOUT * 1000
+                self.props.video_filter = watchdog
+                watchdog = Gst.ElementFactory.make("watchdog", None)
+                watchdog.props.timeout = WATCHDOG_TIMEOUT * 1000
+                self.props.audio_filter = watchdog
+
+    def set_mode(self, mode):
+        self._next_seek = None
+        return GES.Pipeline.set_mode(self, mode)
+
     def _getDuration(self):
         return self._timeline.get_duration()
 
@@ -568,17 +677,20 @@ class Pipeline(GES.Pipeline, SimplePipeline):
         try:
             position = self.getPosition()
         except PipelineError:
-            self.warning("Couldn't get position (you're framestepping too quickly), ignoring this request")
+            self.warning(
+                "Couldn't get position (you're framestepping too quickly), ignoring this request")
             return
 
-        cur_frame = int(round(position * framerate.num / float(Gst.SECOND * framerate.denom), 2))
+        cur_frame = int(
+            round(position * framerate.num / float(Gst.SECOND * framerate.denom), 2))
         new_frame = cur_frame + frames_offset
-        new_pos = int(new_frame * Gst.SECOND * framerate.denom / framerate.num)
+        new_pos = int(new_frame * Gst.SECOND * framerate.denom / framerate.num) + \
+            int((Gst.SECOND * framerate.denom / framerate.num) / 2)
         Loggable.info(self, "From frame %d to %d at %f fps, seek to %s s",
-                    cur_frame,
-                    new_frame,
-                    framerate.num / framerate.denom,
-                    new_pos / float(Gst.SECOND))
+                      cur_frame,
+                      new_frame,
+                      framerate.num / framerate.denom,
+                      new_pos / float(Gst.SECOND))
         self.simple_seek(new_pos)
 
     def _seekCb(self, unused_seeker, position):
@@ -589,3 +701,60 @@ class Pipeline(GES.Pipeline, SimplePipeline):
         end of the timeline.
         """
         self.simple_seek(position)
+
+    def simple_seek(self, position):
+        if self._timeline.is_empty():
+            return
+
+        if self._rendering():
+            raise PipelineError("Trying to seek while rendering")
+
+        st = Gst.Structure.new_empty("seek")
+
+        if self.getState() == Gst.State.PLAYING:
+            st.set_value("playback_time", float(
+                self.getPosition()) / Gst.SECOND)
+
+        st.set_value("start", float(position / Gst.SECOND))
+        st.set_value("flags", "accurate+flush")
+        self.app.write_action(st)
+
+        SimplePipeline.simple_seek(self, position)
+
+    def _busMessageCb(self, bus, message):
+        if message.type == Gst.MessageType.ASYNC_DONE:
+            self.commiting = False
+        if message.type == Gst.MessageType.ASYNC_DONE and\
+                self._commit_wanted:
+            self.debug("Commiting now that ASYNC is DONE")
+            self._addWaitingForAsyncDoneTimeout()
+            self._timeline.commit()
+            self._commit_wanted = False
+        else:
+            super(Pipeline, self)._busMessageCb(bus, message)
+
+    def commit_timeline(self):
+        if self._waiting_for_async_done and not self._was_empty\
+                and not self._timeline.is_empty():
+            self._commit_wanted = True
+            self._was_empty = False
+            self.debug("commit wanted")
+        else:
+            self._addWaitingForAsyncDoneTimeout()
+            self._timeline.commit()
+            self.debug("Commiting right now")
+            if self._timeline.is_empty():
+                self._was_empty = True
+            else:
+                self._was_empty = False
+
+    def setState(self, state):
+        super(Pipeline, self).setState(state)
+        if state >= Gst.State.PAUSED and self._timeline.is_empty():
+            self.debug("No ASYNC_DONE will be emited on empty timelines")
+            self._was_empty = True
+            self._removeWaitingForAsyncDoneTimeout()
+
+    def _rendering(self):
+        mask = GES.PipelineFlags.RENDER | GES.PipelineFlags.SMART_RENDER
+        return self._pipeline.get_mode() & mask
