@@ -35,7 +35,6 @@ import urllib.error
 
 from gettext import ngettext, gettext as _
 
-from gi.repository import Clutter
 from gi.repository import GLib
 from gi.repository import GES
 from gi.repository import Gdk
@@ -47,21 +46,22 @@ from gi.repository.GstPbutils import DiscovererVideoInfo, DiscovererAudioInfo,\
 
 from pitivi.utils.loggable import doLog, ERROR
 from pitivi.utils.misc import path_from_uri
+from pitivi.configure import get_pixmap_dir
 
 
 # Dimensions in pixels
 TRACK_SPACING = 8
 EXPANDED_SIZE = 65
-CONTROL_WIDTH = 250
+CONTROL_WIDTH = 240
 
 PADDING = 6
 SPACING = 10
 
 PLAYHEAD_WIDTH = 1
+SNAPBAR_WIDTH = 5
 CANVAS_SPACING = 21
 KEYFRAME_SIZE = 8
-
-PLAYHEAD_COLOR = Clutter.Color.new(200, 0, 0, 255)
+LAYER_HEIGHT = 130
 
 # Layer creation blocking time in s
 LAYER_CREATION_BLOCK_TIME = 0.2
@@ -69,9 +69,7 @@ LAYER_CREATION_BLOCK_TIME = 0.2
 # Drag and drop
 FILE_TARGET_ENTRY = Gtk.TargetEntry.new("text/plain", 0, 0)
 URI_TARGET_ENTRY = Gtk.TargetEntry.new("text/uri-list", 0, 0)
-FILESOURCE_TARGET_ENTRY = Gtk.TargetEntry.new("pitivi/file-source", 0, 0)
 EFFECT_TARGET_ENTRY = Gtk.TargetEntry.new("pitivi/effect", 0, 0)
-LAYER_CONTROL_TARGET_ENTRY = Gtk.TargetEntry.new("pitivi/layer-control", 0, 0)
 
 
 def _get_settings(schema):
@@ -97,9 +95,85 @@ NORMAL_FONT = _get_font("font-name", "Cantarell")
 DOCUMENT_FONT = _get_font("document-font-name", "Sans")
 MONOSPACE_FONT = _get_font("monospace-font-name", "Monospace")
 
+TIMELINE_CSS = """
+    .AudioBackground {
+        background-color: #496c21;
+    }
 
+    .VideoBackground {
+        background-color: #2d2d2d;
+    }
+
+    .AudioBackground:selected {
+        background-color: #1b2e0e;
+    }
+
+    .VideoBackground:selected {
+        background-color: #0f0f0f;
+    }
+
+    .Trimbar {
+        background-image: url('%(trimbar_normal)s');
+        opacity:0.5;
+    }
+
+    .Trimbar.left {
+        border-radius: 5px 0px 0px 5px;
+    }
+
+    .Trimbar.right {
+        border-radius: 0px 5px 5px 0px;
+    }
+
+    .Trimbar:hover {
+         background-image: url('%(trimbar_focused)s');
+    }
+
+    .PlayHead {
+        background-color: red;
+    }
+
+    .Clip {
+    /* TODO */
+    }
+
+    .SnapBar {
+        background-color: rgb(127, 153, 204);
+    }
+
+    .TransitionClip {
+         background-color: rgba(127, 153, 204, 0.5);
+    }
+
+    .TransitionClip:selected {
+         background-color: rgba(127, 200, 204, 0.7);
+    }
+
+    .SpacedSeparator:hover {
+        background-color: rgba(127, 153, 204, 0.5);
+        opacity:0.5;
+    }
+
+    .SpacedSeparator {
+        background-color: rgba(0, 0, 0, 0.10);
+        border-bottom-color: rgba(0, 0, 0, 0.10);
+        border-left-color: rgba(0, 0, 0, 0.10);
+        border-right-color: rgba(0, 0, 0, 0.10);
+        border-top-color: rgba(0, 0, 0, 0.10);
+        color: rgba(0, 0, 0, 0.10);
+        outline-color: rgba(238, 238, 236, 0.29);
+        opacity:1;
+    }
+
+    .Marquee {
+         background-color: rgba(224, 224, 224, 0.7);
+    }
+
+""" % ({'trimbar_normal': os.path.join(get_pixmap_dir(), "trimbar-normal.png"),
+        'trimbar_focused': os.path.join(get_pixmap_dir(), "trimbar-focused.png")})
 # ---------------------- ARGB color helper-------------------------------------#
 
+
 def argb_to_gdk_rgba(color_int):
     return Gdk.RGBA(color_int / 256 ** 2 % 256 / 255.,
                     color_int / 256 ** 1 % 256 / 255.,
@@ -164,16 +238,14 @@ def hex_to_rgb(value):
 
 
 def set_cairo_color(context, color):
-    if type(color) is Clutter.Color:
-        color = (color.red, color.green, color.blue)
-
     if type(color) is Gdk.RGBA:
         cairo_color = (float(color.red), float(color.green), float(color.blue))
     elif type(color) is tuple:
         # Cairo's set_source_rgb function expects values from 0.0 to 1.0
         cairo_color = [max(0, min(1, x / 255.0)) for x in color]
     else:
-        raise Exception("Unexpected color parameter: %s, %s" % (type(color), color))
+        raise Exception("Unexpected color parameter: %s, %s" %
+                        (type(color), color))
     context.set_source_rgb(*cairo_color)
 
 
@@ -207,7 +279,7 @@ def beautify_info(info):
             nice_streams_txts.append(beautified_string)
 
     return ("<b>" + path_from_uri(info.get_uri()) + "</b>\n" +
-        "\n".join(nice_streams_txts))
+            "\n".join(nice_streams_txts))
 
 
 def info_name(info):
@@ -227,11 +299,12 @@ def info_name(info):
 
 def beautify_stream(stream):
     if type(stream) is DiscovererAudioInfo:
-        templ = ngettext("<b>Audio:</b> %d channel at %d <i>Hz</i> (%d <i>bits</i>)",
-                "<b>Audio:</b> %d channels at %d <i>Hz</i> (%d <i>bits</i>)",
-                stream.get_channels())
+        templ = ngettext(
+            "<b>Audio:</b> %d channel at %d <i>Hz</i> (%d <i>bits</i>)",
+            "<b>Audio:</b> %d channels at %d <i>Hz</i> (%d <i>bits</i>)",
+            stream.get_channels())
         templ = templ % (stream.get_channels(), stream.get_sample_rate(),
-            stream.get_depth())
+                         stream.get_depth())
         return templ
 
     elif type(stream) is DiscovererVideoInfo:
@@ -240,9 +313,10 @@ def beautify_stream(stream):
             templ = _("<b>Video:</b> %d×%d <i>pixels</i> at %.3f <i>fps</i>")
             try:
                 templ = templ % (par * stream.get_width(), stream.get_height(),
-                    float(stream.get_framerate_num()) / stream.get_framerate_denom())
+                                 float(stream.get_framerate_num()) / stream.get_framerate_denom())
             except ZeroDivisionError:
-                templ = templ % (par * stream.get_width(), stream.get_height(), 0)
+                templ = templ % (
+                    par * stream.get_width(), stream.get_height(), 0)
         else:
             templ = _("<b>Image:</b> %d×%d <i>pixels</i>")
             templ = templ % (par * stream.get_width(), stream.get_height())
@@ -394,7 +468,24 @@ def alter_style_class(style_class, target_widget, css_style):
     toolbar_css = "%s { %s }" % (style_class, css_style)
     css_provider.load_from_data(toolbar_css.encode('UTF-8'))
     style_context = target_widget.get_style_context()
-    style_context.add_provider(css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
+    style_context.add_provider(
+        css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
+
+
+def set_children_state_recurse(widget, state):
+    widget.set_state_flags(state, False)
+    for child in widget.get_children():
+        child.set_state_flags(state, False)
+        if isinstance(child, Gtk.Container):
+            set_children_state_recurse(child, state)
+
+
+def unset_children_state_recurse(widget, state):
+    widget.unset_state_flags(state)
+    for child in widget.get_children():
+        child.unset_state_flags(state)
+        if isinstance(child, Gtk.Container):
+            unset_children_state_recurse(child, state)
 
 
 # ----------------------- encoding datas --------------------------------------- #
