@@ -28,6 +28,7 @@ from pitivi.utils.system import System, getSystem, GnomeSystem, \
 
 
 class TestSystem(TestCase):
+
     def setUp(self):
         self.system = System()
 
@@ -41,17 +42,17 @@ class TestSystem(TestCase):
         self.assertEqual("a b", self.system.getUniqueFilename("a b"))
 
     def testScreensaverInhibit(self):
-        #check that we start of uninhibited
+        # check that we start of uninhibited
         self.assertTrue(not self.system.screensaverIsInhibited())
 
-        #inhibit and check that we are
+        # inhibit and check that we are
         self.system.inhibitScreensaver("a")
         self.assertTrue(self.system.screensaverIsInhibited())
-        #uninhibit and check that we are
+        # uninhibit and check that we are
         self.system.uninhibitScreensaver("a")
         self.assertTrue(not self.system.screensaverIsInhibited())
 
-        #check that adding/removing is consistent with multiple keys
+        # check that adding/removing is consistent with multiple keys
         for c in range(0, 5):
             self.system.inhibitScreensaver(str(c))
             self.assertTrue(self.system.screensaverIsInhibited(str(c)))
@@ -63,17 +64,17 @@ class TestSystem(TestCase):
         self.assertTrue(not self.system.screensaverIsInhibited())
 
     def testSleepInhibit(self):
-        #check that we start of uninhibited
+        # check that we start of uninhibited
         self.assertTrue(not self.system.sleepIsInhibited())
 
-        #inhibit and check that we are
+        # inhibit and check that we are
         self.system.inhibitSleep("a")
         self.assertTrue(self.system.sleepIsInhibited())
-        #uninhibit and check that we are
+        # uninhibit and check that we are
         self.system.uninhibitSleep("a")
         self.assertTrue(not self.system.sleepIsInhibited())
 
-        #check that adding/removing is consistent with multiple keys
+        # check that adding/removing is consistent with multiple keys
         for c in range(0, 5):
             self.system.inhibitSleep(str(c))
             self.assertTrue(self.system.sleepIsInhibited(str(c)))
@@ -95,7 +96,7 @@ class TestGnomeSystem(TestCase):
             # We can only test this on a Gnome system.
             return
 
-        if not self.system.session_iface.IsInhibited(
+        if self.system.session_iface.IsInhibited(
                 INHIBIT_LOGOUT | INHIBIT_USER_SWITCHING | INHIBIT_SUSPEND |
                 INHIBIT_SESSION_IDLE):
             # Other programs are inhibiting, cannot test.
