@@ -1,4 +1,4 @@
-#!/usr/bin/python2
+#!/usr/bin/python3
 # Pitivi video editor
 #
 #       test_utils.py
@@ -25,7 +25,7 @@ from unittest import TestCase
 
 from gi.repository import Gst
 from pitivi.utils.ui import beautify_length
-from pitivi.check import *
+from pitivi.check import *  # noqa
 
 second = Gst.SECOND
 minute = second * 60
@@ -33,6 +33,7 @@ hour = minute * 60
 
 
 class TestBeautifyLength(TestCase):
+
     def testBeautifySeconds(self):
         self.assertEqual(beautify_length(second), "1 second")
         self.assertEqual(beautify_length(second * 2), "2 seconds")
@@ -47,20 +48,21 @@ class TestBeautifyLength(TestCase):
 
     def testBeautifyMinutesAndSeconds(self):
         self.assertEqual(beautify_length(minute + second),
-                "1 minute, 1 second")
+                         "1 minute, 1 second")
 
     def testBeautifyHoursAndMinutes(self):
         self.assertEqual(beautify_length(hour + minute + second),
-                "1 hour, 1 minute")
+                         "1 hour, 1 minute")
 
 
 class TestDependencyChecks(TestCase):
+
     def testDependencies(self):
-        gi_dep = GstDependency("Gst", "1.0.0")
+        gi_dep = GstDependency("Gst", "1.0", "1.0.0")
         gi_dep.check()
         self.assertTrue(gi_dep.satisfied)
 
-        gi_dep = GstDependency("Gst", "9.9.9")
+        gi_dep = GstDependency("Gst", "1.0", "9.9.9")
         gi_dep.check()
         self.assertFalse(gi_dep.satisfied)
 
@@ -68,11 +70,11 @@ class TestDependencyChecks(TestCase):
         gi_dep.check()
         self.assertFalse(gi_dep.satisfied)
 
-        gi_dep = GtkOrClutterDependency("Gtk", "3.0.0")
+        gi_dep = GtkDependency("Gtk", "3.0", "3.0.0")
         gi_dep.check()
         self.assertTrue(gi_dep.satisfied)
 
-        gi_dep = GtkOrClutterDependency("Gtk", "9.9.9")
+        gi_dep = GtkDependency("Gtk", "3.0", "9.9.9")
         gi_dep.check()
         self.assertFalse(gi_dep.satisfied)
 
@@ -87,11 +89,3 @@ class TestDependencyChecks(TestCase):
         classic_dep = ClassicDependency("numpy", None)
         classic_dep.check()
         self.assertTrue(classic_dep.satisfied)
-
-        gst_plugin_dep = GstPluginDependency("gnonlin", "1.1.90")
-        gst_plugin_dep.check()
-        self.assertTrue(gst_plugin_dep.satisfied)
-
-        gst_plugin_dep = GstPluginDependency("gnonlin", "9.9.9")
-        gst_plugin_dep.check()
-        self.assertFalse(gst_plugin_dep.satisfied)
