@@ -17,11 +17,12 @@
 # Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 # Boston, MA 02110-1301, USA.
 
-import mock
-from unittest import TestCase
+from unittest import TestCase, mock
 
 from gi.repository import GES
 
+from tests import common
+
 from pitivi.utils.timeline import Selected, Selection, SELECT, SELECT_ADD, \
     UNSELECT
 
@@ -54,8 +55,8 @@ class TestSelection(TestCase):
 
     def testGetSingleClip(self):
         selection = Selection()
-        clip1 = GES.UriClip()
-        clip2 = GES.TitleClip()
+        clip1 = common.createTestClip(GES.UriClip)
+        clip2 = common.createTestClip(GES.TitleClip)
 
         # Selection empty.
         self.assertFalse(selection.getSingleClip(GES.TitleClip))
